{$Mode Delphi}
unit InfoImageU;

interface

uses
  Windows, LMessages, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls;

type

  TInfoImageFlag = (iifAutoSize, // Setzt die Größe des HintWindows automatisch auf die größte
                    iifRes1, iifRes2, iifRes3, iifRes4, iifRes5, iifRes6, iifRes7, iifRes8);
  TInfoImageFlags = set of TInfoImageFlag;

  TGetAutoSizeEvent = procedure (Sender : TObject; var aWidth, aHeight : integer) of object;

  TInfoImage = class ( TImage )
  private
   fHintWindow : TForm;
   fOnMouseEnter : TNotifyEvent;
   fOnMouseLeave : TNotifyEvent;
   fOnHintWindowPaint : TNotifyEvent;
   fHintWindowWidth,
   fHintWindowHeight : integer;
   fFlags : TInfoImageFlags;
   fOnGetAutoSize : TGetAutoSizeEvent;
   procedure LMMouseEnter(var Message : TMessage); message CM_MouseEnter;
   procedure LMMouseLeave(var Message : TMessage); message CM_MouseLeave;
   function  GetFlag(Index : Integer) : boolean;
   procedure SetFlag(Index : Integer; Value : boolean);
  protected
   procedure HintWindowPaint(Sender : TObject);
   property  HintWindow : TForm read fHintWindow write fHintWindow;
  public
   constructor Create(AOwner : TComponent); override;
   destructor  destroy; override;
  published
   property  HintwindowAutoSize : boolean Index 0 read GetFlag write SetFlag;
   property  OnHintWindowPaint : TNotifyEvent read fOnHintWindowPaint write fOnHintWindowPaint;
   property  HintWindowWidth  : Integer read fHintWindowWidth write fHintWindowWidth stored true default 70;
   property  HintWindowHeight : Integer read fHintWindowHeight write fHintWindowHeight stored true default 70;
   property  OnMouseEnter : TNotifyEvent read fOnMouseEnter write fOnMouseEnter;
   property  OnMouseLeave : TNotifyEvent read fOnMouseLeave write fOnMouseLeave;
   property  OnGetAutoSize : TGetAutoSizeEvent read fOnGetAutoSize write fOnGetAutoSize;
  end;

  TTextInfoImage = class (TInfoImage)
  private
   fText : TStringlist;
  protected
   procedure DoAutoSize(Sender : TObject; var aWidth, aHeight : integer); reintroduce;
   procedure CMMouseEnter(var Message : TMessage); message CM_MouseEnter;
   procedure DoHintWindowPaint(Sender : TObject);
  public
   constructor Create(AOwner : TComponent); override;
   destructor  Destroy; override;
   property    Text : TStringlist read fText write fText;
  end;

  TGrafikInfoImage = class ( TInfoImage )
  private
   fImageGraphic : TPicture;
  protected
   procedure   DoAutoSize(Sender : TObject; var aWidth, aHeight : integer); reintroduce;
   procedure   CMMouseEnter(var Message : TMessage); message CM_MouseEnter;
   procedure   DoHintwindowPaint(Sender : TObject);
  public
   constructor Create(AOwner : TComponent); override;
   destructor  Destroy; override;
   property    fHintWindowPicture : TPicture read fImageGraphic write fImageGraphic;
  end;


procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Misc', [TInfoImage, TTextInfoImage, TGrafikInfoImage]);
end;

// Das Verhältnis von w zu h muß erhalten bleiben, das Ergebnis muss R passen
function ScaleRect(R : TRect; iw, ih : integer) : TRect;
var
	h, w, dh, dw: double;
        dx, dy : integer;
begin
        dx := r.Left;
        dy := r.Top;
        R.Left := 0;
        R.Top  := 0;
        dec(R.Right, dx);
        dec(R.Bottom, dy);


	h :=  ih;
	w :=  iw;
	dh := h / (R.Bottom-R.Top);
	dw := w / (R.Right-R.Left);

   Result := R;

   if ((h=0) or (w=0) or (dh=0) or (dw=0)) then exit;

   if ((dh>1) and (dh>dw)) then
   begin
      Result.Bottom := Result.Top + trunc(h /dh);
      Result.Right  := Result.Left + trunc(w / dh);
   end else
   if (dw>1) and (dw > dh) then
   begin
      Result.Right  := Result.Left + trunc(w / dw);
      Result.Bottom := Result.Top + trunc(h / dw);
   end
   else
   begin
      if (dh>dw) then
      begin
         Result.Bottom := Result.Top + trunc(h / dh);
         Result.Right := Result.Left + trunc(w / dh);
      end
      else
      begin
         Result.Right  := Result.Left + trunc(w / dw);
         Result.Bottom := Result.Top + trunc(h / dw);
      end
   end;

  iw := (R.Right - R.Left) shr 1 - (Result.Right - Result.Left) shr 1;
  ih := (R.Bottom - R.Top) shr 1 - (Result.Bottom - Result.top) shr 1;

  // Verschieben des Bereiches in die Mitte

  with Result do
  begin
   Left := Left + iw + dx;
   Right := Right + iw + dx;
   Top := Top + ih + dy;
   Bottom := Bottom + ih + dy;
  end;
end;


{ TInfoImage }

procedure TInfoImage.LMMouseEnter(var Message: TMessage);
var Pu, po : TPoint;
    w, h : integer;
begin
 if not Assigned(fHintWindow) then
 begin
  fHintWindow := TForm.Create(nil);
 end;
 fHintWindow.OnPaint := HintWindowPaint;
 po := ClientToScreen(Point(0, -2));
 pu := ClientToScreen(Point(0, Height+2));
 with fHintWindow do
 begin
  Color := clWhite;
  ControlStyle := [csNoStdEvents];
  BorderStyle := bsNone;
  FormStyle := fsStayOnTop;

  // Hier ging's bislang immer schief. Gehen wir mal Schritt für Schritt weiter
  // Als erstes ermitteln wir, wohin wir uns am weitesten Ausdehnen können

  Width := fHintWindowWidth;
  Height := fHintWindowHeight;
  w := Width;
  h := Height;
  if Assigned(fOnGetAutoSize) then fOnGetAutoSize(fHintWindow, w, h);
  if h <> Height then Height := h;
  if w <> Width then Width := w;
  if Width > Screen.Width then width := Screen.Width;
  Left := Pu.x;
  if Left < 0 then Left := 0;
  if Left + Width > Screen.Width then Left := Screen.Width - Width;
  Top := Pu.Y;
  if Top + Height > Screen.Height then Top := Pu.y - Height - self.Height;
//   Visible := True;
  if Top < 0 then
  begin
   Height := Height + Top;
   Top := 0;
  end;

  ShowWindow(Handle, SW_SHOWNOACTIVATE);
 end;
 if Assigned(fOnMouseEnter) then fOnMouseEnter(self);
end;

procedure TInfoImage.LMMouseLeave(var Message: TMessage);
begin
  if Assigned(fOnMouseLeave) then fOnMouseLeave(self);
  if Assigned(fHintWindow) then
  begin
   ShowWindow(fHintWindow.Handle, sw_hide);
  end;
end;

constructor TInfoImage.Create(AOwner: TComponent);
begin
 inherited;
 fHintWindow := NIL;
// fHintWindowWidth := 200;
// fHintWindowHeight := 150;
end;

destructor TInfoImage.destroy;
begin
  if Assigned(fHintWindow) then fHintWindow.Free;
  inherited;
end;

function TInfoImage.GetFlag(Index: Integer): boolean;
begin
 Result := TInfoImageFlag(Index) in fFlags;
end;

procedure TInfoImage.HintWindowPaint(Sender: TObject);
begin
 if Assigned(fOnHintWindowPaint) then fOnHintWindowPaint(fHintWindow);
end;

procedure TInfoImage.SetFlag(Index: Integer; Value: boolean);
begin
 if GetFlag(Index) <> Value then
 begin
  if Value then Include(fFlags, TInfoImageFlag(Index)) else Exclude(fFlags, TInfoImageFlag(Index));
 end;
end;

{ TTextInfoImage }

procedure TTextInfoImage.CMMouseEnter(var Message: TMessage);
begin
 if (Assigned(fText)) and (fText.Count > 0) then inherited;
end;

constructor TTextInfoImage.Create(AOwner: TComponent);
begin
 inherited;
 fText := TStringlist.Create;
 OnGetAutoSize := DoAutoSize;
 OnHintWindowPaint := DoHintWindowPaint;
end;

destructor TTextInfoImage.Destroy;
begin
 fText.Free;
 inherited;
end;

procedure TTextInfoImage.DoAutoSize(Sender: TObject; var aWidth,
  aHeight: integer);
var hw : TForm;
    l, largest : Integer;
    i : integer;

begin
 hw := Sender as TForm;
 if fText.Count > 0 then
 begin
  Largest := 0;
  for i := 0 to Pred(fText.Count) do
  begin
   l := hw.Canvas.TextWidth(fText[i]);
   if l > Largest then Largest := l;
  end;
  l := hw.Canvas.TextHeight('A');
  aWidth := Largest + 10;
  aHeight := l * fText.Count + 10;
 end;
end;

procedure TTextInfoImage.DoHintWindowPaint(Sender: TObject);
var R : TRect;
begin
// inherited;
 with TForm(Sender).Canvas do
 begin
  Brush.Color := $DEF7F7;
  Pen.Color := clBlack;
  Rectangle(TForm(Sender).ClientRect);
 end;
 R := TForm(Sender).ClientRect;

 R.Left := 5;
 R.Top := 5;
 R.Right := R.Right + 5;
 R.Bottom := R.Bottom + 5;
 DrawText(TForm(Sender).Canvas.Handle, PChar(fText.Text), Length(fText.Text), R, DT_WORDBREAK);
end;

{ TGrafikInfoImage }

procedure TGrafikInfoImage.CMMouseEnter(var Message: TMessage);
begin
 if (Assigned(fImageGraphic)) and (Assigned(fImageGraphic.Graphic)) then
 begin
  if fImageGraphic.Graphic.Width < HintWindowWidth then HintWindowWidth  := fImageGraphic.Graphic.Width;
  if fImageGraphic.Graphic.Height < HintWindowHeight then HintWindowHeight := fImageGraphic.Graphic.Height;
 end;
 inherited;
end;

constructor TGrafikInfoImage.Create(AOwner: TComponent);
begin
 inherited;
 fImageGraphic := TPicture.Create;
 OnGetAutoSize := DoAutoSize;
 OnHintWindowPaint := DoHintWindowPaint;
end;

destructor TGrafikInfoImage.Destroy;
begin
  fImageGraphic.Free;
  inherited;
end;

procedure TGrafikInfoImage.DoAutoSize(Sender: TObject; var aWidth,
  aHeight: integer);
begin
 if AutoSize then
 begin
  if (Assigned(fImageGraphic)) and (Assigned(fImageGraphic.Graphic)) then
  begin
   aWidth := fImageGraphic.Width;
   aHeight := fImageGraphic.Height;
  end;
 end else
 begin
  aWidth := HintWindowWidth;
  aHeight := HintWindowHeight;
 end;
end;

procedure TGrafikInfoImage.DoHintwindowPaint(Sender: TObject);
var R : TRect;
begin
 if not Assigned(fImageGraphic) then exit;
 if not Assigned(fImageGraphic.Graphic) then exit;
 with TForm(Sender) do
 begin
  if (Height < fImageGraphic.Graphic.Height) or
     (Width < fImageGraphic.Graphic.Width) then
  begin
   R := ClientRect;
   Canvas.Rectangle(R);
   inc(r.left); inc(r.top);
   dec(r.right); dec(r.bottom);
   R := ScaleRect(R, fImageGraphic.Graphic.Width, fImageGraphic.Graphic.Height);
   //StretchBlt(Canvas.Handle, R.Left, R.Top, R.Right - R.Left, R.Bottom - R.Top, fImageGraphic.Graphic.Handle, 0, 0, fImageGraphic.Graphic.Width, fImageGraphic.Graphic.Height);
   Canvas.StretchDraw(R, fImageGraphic.Graphic);
  end else
   Canvas.Draw(0, 0, fImageGraphic.Graphic);
 end;
end;

end.
